/*
 * main.c
 *
 *  Created on: 19 lip 2015
 *      Author: Szymon
 */

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/iotn13.h>
#include <avr/sleep.h>
#include <inttypes.h>

#define PIN_RELAY 4
#define PIN_BUTTON 3
#define FILTER 10
#define ACTIVE 188

static inline uint8_t button_value(void) {
	if (PINB & (1 << PIN_BUTTON)) {
		return 0;
	}
	return 1;
}

static inline void relay_set(uint8_t value) {
	if (value) {
		PORTB |= 1 << PIN_RELAY;
	} else {
		PORTB &= ~(1 << PIN_RELAY);
	}
}

ISR(TIM0_COMPA_vect) {
	static uint8_t div = 0;
	static uint8_t filter = FILTER;
	static uint8_t active = 0;

	if (button_value()) {
		if (filter) {
			filter--;
			if (!filter) {
				div++;
				if (div >= 4) {
					div = 0;
					active = ACTIVE;
					relay_set(1);
				}
			}
		}
	} else {
		filter = FILTER;
	}

	if (active) {
		active--;
		if (!active) {
			relay_set(0);
		}
	}
}

int main(void) {
	DDRB = 1 << PIN_RELAY;
	PORTB = 0x3F & (~(1 << PIN_RELAY));


	OCR0A = 94;							/*100Hz, at 10ms*/
	TCCR0A = 1 << WGM01;
	TCCR0B = (1 << CS01) | (1 << CS00);	/*OSC 4.8MHz divide by 8 internally,
	 	 	 	 	 	  	  	  	  	  divide by 64, give 512*/

	TIMSK0 |= 1 << OCIE0A;

	set_sleep_mode(SLEEP_MODE_IDLE);
	sei();
	while (1) {
        sleep_enable();
        sleep_cpu();
        sleep_disable();
	}
	return 0;
}
